"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/anthropic/anthropic.index.ts
var anthropic_index_exports = {};
__export(anthropic_index_exports, {
  AnthropicRunner: () => AnthropicRunner,
  runner: () => runner
});
module.exports = __toCommonJS(anthropic_index_exports);

// src/anthropic/anthropic.runner.ts
var import_sdk = require("@anthropic-ai/sdk");
var import_resolved3 = require("@synstack/resolved");
var import_zod_to_json_schema = require("zod-to-json-schema");

// ../shared/src/ts.utils.ts
function never(neverCheck, value) {
  throw new Error(
    `Unexpected value, received: ${JSON.stringify(value ?? neverCheck, null, 2)}`
  );
}

// src/completion.builder.ts
var import_json = require("@synstack/json");
var import_resolved2 = require("@synstack/resolved");
var import_text = require("@synstack/text");

// src/completion.runner.ts
var import_resolved = require("@synstack/resolved");

// src/llm.types.ts
var import_zod = require("zod");

// src/completion.builder.ts
var CompletionBuilder2 = class _CompletionBuilder {
  _completion;
  constructor(completion2) {
    this._completion = completion2;
  }
  static get new() {
    return new _CompletionBuilder({});
  }
  static from(completion2) {
    return _CompletionBuilder.new.merge(completion2);
  }
  get $() {
    return this._completion;
  }
  $_(fn) {
    return (0, import_resolved2.pipe)(this.$)._(fn);
  }
  async toSync() {
    return new _CompletionBuilder(await this.$);
  }
  /**
   * Override the values of the current builder with the provided values as object
   */
  merge(completion2) {
    return new _CompletionBuilder(
      (0, import_resolved2.pipe)([this._completion, completion2])._(([old, added]) => ({
        ...old,
        ...added
      })).$
    );
  }
  // Todo: allow calling as a template directly
  /**
   * Set the system prompt
   */
  system(system) {
    return this.merge(
      (0, import_resolved2.pipe)(system)._((system2) => ({
        system: system2
      })).$
    );
  }
  /**
   * Set the llm temperature
   * @argument temperature The temperature value between 0 and 1
   */
  temperature(temperature) {
    return this.merge(
      (0, import_resolved2.pipe)(temperature)._((temperature2) => ({
        temperature: temperature2
      })).$
    );
  }
  /**
   * Set the maximum number of tokens to generate
   */
  maxTokens(maxTokens) {
    return this.merge(
      (0, import_resolved2.pipe)(maxTokens)._((maxTokens2) => ({ maxTokens: maxTokens2 })).$
    );
  }
  /**
   * Set the stop sequences.
   *
   * If the LLM generates a stop sequence, the completion will stop.
   */
  stopSequences(sequences) {
    return this.merge(
      (0, import_resolved2.pipe)(sequences)._((sequences2) => ({
        stopSequences: sequences2
      })).$
    );
  }
  /**
   * Set the top k tokens to consider for the completion
   */
  topK(topK) {
    return this.merge(
      (0, import_resolved2.pipe)(topK)._((topK2) => ({
        topK: topK2
      })).$
    );
  }
  /**
   * Set the top p tokens to consider for the completion
   */
  topP(topP) {
    return this.merge(
      (0, import_resolved2.pipe)(topP)._((topP2) => ({
        topP: topP2
      })).$
    );
  }
  /**
   * Reset the tools
   */
  clearTools() {
    return this.merge({
      toolsConfig: void 0
    });
  }
  /**
   * Set the tools available to the LLM
   *
   * @argument tools The tools to use
   * @argument requireToolUse Whether to require the LLM to use the tools
   */
  tools(tools, requireToolUse = false) {
    return this.merge(
      (0, import_resolved2.pipe)(tools)._((tools2) => ({
        toolsConfig: {
          type: "multi",
          tools: tools2,
          requireToolUse
        }
      })).$
    );
  }
  /**
   * Set a single tool for the LLM to use. The LLM will always respond with this tool
   */
  tool(tool) {
    return this.merge(
      (0, import_resolved2.pipe)(tool)._((tool2) => ({
        toolsConfig: {
          type: "single",
          tool: tool2
        }
      })).$
    );
  }
  /**
   * Set the stop reason for the completion
   */
  stopReason(stopReason) {
    return this.merge(
      (0, import_resolved2.pipe)(stopReason)._((stopReason2) => ({
        stopReason: stopReason2
      })).$
    );
  }
  /**
   * Set the tokens usage
   */
  usage(usage) {
    return this.merge(
      (0, import_resolved2.pipe)(usage)._((usage2) => ({
        usage: usage2
      })).$
    );
  }
  /**
   * Add usage to existing usage
   */
  addUsage(usage) {
    return this.merge(
      (0, import_resolved2.pipe)([this.$, usage])._(([$, usage2]) => ({
        usage: {
          inputTokens: $.usage?.inputTokens ?? 0 + usage2.inputTokens,
          outputTokens: $.usage?.outputTokens ?? 0 + usage2.outputTokens
        }
      })).$
    );
  }
  /**
   * Set messages
   */
  messages(messages) {
    return this.merge(
      (0, import_resolved2.pipe)(messages)._((messages2) => ({
        messages: messages2
      })).$
    );
  }
  /**
   * Add messages
   */
  addMessages(messages) {
    return this.merge(
      (0, import_resolved2.pipe)([this.$, messages])._(([$, messages2]) => ({
        messages: [...$.messages ?? [], ...messages2]
      })).$
    );
  }
  /**
   * Add assistant message
   */
  addAssistantMessage(message) {
    return this.merge(
      (0, import_resolved2.pipe)([this.$, message])._(([$, message2]) => {
        const $messages = $.messages ?? [];
        const lastMessage = $.messages.at(-1);
        if (!lastMessage)
          return {
            messages: [message2]
          };
        if (lastMessage.role !== "assistant")
          return {
            messages: [...$messages, message2]
          };
        return {
          messages: [
            ...$messages.slice(0, -1),
            {
              ...lastMessage,
              content: [...lastMessage.content, ...message2.content]
            }
          ]
        };
      }).$
    );
  }
  /**
   * Execute completion on Llm runner
   */
  run(runner2) {
    return runner2.runChatCompletion(this);
  }
  lastMessage() {
    return (0, import_resolved2.pipe)(this.$)._((completion2) => {
      const lastMessage = completion2.messages?.at(-1);
      if (!lastMessage) throw new EmptyMessagesException(completion2);
      return lastMessage;
    }).$;
  }
  /**
   * Get last message as text
   */
  lastMessageText() {
    return (0, import_resolved2.pipe)(this.lastMessage())._(
      (message) => message.content.filter((c) => c.type === "text").map((c) => c.text).join("")
    ).$;
  }
  /**
   * Get last message tool calls
   */
  lastMessageToolCalls() {
    return (0, import_resolved2.pipe)(this.lastMessage())._((message) => {
      if (message.role !== "assistant") return [];
      return message.content.filter((c) => c.type === "tool_call");
    }).$;
  }
};
var EmptyMessagesException = class extends Error {
  constructor(completion2) {
    super(import_text.t`
      No messages in current completion state:
        ${import_json.json.serialize(completion2, { pretty: true })} 
    `);
  }
};
var completion = CompletionBuilder2.new;

// src/anthropic/anthropic.runner.ts
var AnthropicRunner = class _AnthropicRunner {
  // implements CompletionRunner<AnthropicRunner.Completion>
  _config;
  static MODELS = {
    CLAUDE_3_SMALL: "claude-3-haiku-20240307",
    CLAUDE_3_MEDIUM: "claude-3-sonnet-20240229",
    CLAUDE_3_LARGE: "claude-3-opus-20240229",
    CLAUDE_3_5_MEDIUM: "claude-3-5-sonnet-latest"
  };
  // #region build
  constructor(config) {
    this._config = {
      ...config,
      continues: config.continues ?? 0,
      retries: config.retries ?? 0
    };
  }
  static new() {
    return new _AnthropicRunner({
      retries: 0,
      continues: 0
    });
  }
  static from(config) {
    return new _AnthropicRunner({
      continues: 0,
      retries: 0,
      ...config
    });
  }
  client(client) {
    return new _AnthropicRunner({ ...this._config, client });
  }
  retries(retries) {
    return new _AnthropicRunner({
      ...this._config,
      retries
    });
  }
  cache(cache) {
    return new _AnthropicRunner({ ...this._config, cache });
  }
  continues(continues) {
    return new _AnthropicRunner({
      ...this._config,
      continues
    });
  }
  retryBackoff(retryBackoff) {
    return new _AnthropicRunner({
      ...this._config,
      retryBackoff
    });
  }
  model(model) {
    return new _AnthropicRunner({ ...this._config, model });
  }
  // #endregion
  // #region run
  async chatCompletion(completion2) {
    const rawCompletion = await completion2.$;
    const anthropicCompletion = this.mapCompletionQuery(rawCompletion);
    const query = this._config.cache ? this._config.cache.fn(
      // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
      this._config.client.messages.create.bind(
        this._config.client.messages
      )
    ) : this._config.client.messages.create.bind(this._config.client.messages);
    const res = await query(
      anthropicCompletion
    );
    const suffix = res.stop_sequence;
    let message = _AnthropicRunner.mapResponseMessage(res);
    const lastContent = message.content.at(-1);
    if (suffix && lastContent?.type === "text") {
      message = {
        ...message,
        content: [
          ...message.content.slice(0, -1),
          { ...lastContent, text: lastContent.text + suffix }
        ]
      };
    }
    const usage = {
      inputTokens: res.usage.input_tokens,
      outputTokens: res.usage.output_tokens
    };
    return {
      message,
      stopReason: _AnthropicRunner.mapStopReason(res.stop_reason),
      usage
    };
  }
  async runChatCompletion(completion2) {
    try {
      const { message, usage, stopReason } = await this.chatCompletion(completion2);
      const outputCompletion = completion2.addAssistantMessage(message).addUsage(usage).stopReason(stopReason);
      if (stopReason === "max_tokens") {
        if (this._config.continues === 0) return outputCompletion.toSync();
        return this.continues(this._config.continues - 1).runChatCompletion(
          outputCompletion
        );
      }
      return outputCompletion.toSync();
    } catch (error) {
      if (this._config.retries === 0) throw error;
      if (this._config.retryBackoff && this._config.retryBackoff > 0)
        await new Promise(
          (resolve) => setTimeout(resolve, this._config.retryBackoff)
        );
      return this.retries(this._config.retries - 1).runChatCompletion(
        completion2
      );
    }
  }
  // #endregion
  // #region map Anthropic -> Domain
  static mapStopReason(reason) {
    if (reason === "end_turn") return "end";
    if (reason === "max_tokens") return "max_tokens";
    if (reason === "tool_use") return "tool_call";
    if (reason === "stop_sequence") return "stop_sequence";
    if (reason === null) return "end";
    never(reason);
  }
  static mapResponseMessage(message) {
    return {
      role: "assistant",
      content: message.content.map(_AnthropicRunner.mapResponseContent)
    };
  }
  static mapResponseContent(content) {
    if (content.type === "text") {
      return {
        type: "text",
        text: content.text
      };
    }
    if (content.type === "tool_use") {
      return {
        type: "tool_call",
        toolCallId: content.id,
        toolName: content.name,
        toolArgs: content.input
      };
    }
    never(content);
  }
  // #endregion
  // #region map Domain -> Anthropic
  mapCompletionQuery(query) {
    return {
      model: this._config.model,
      system: query.system,
      stop_sequences: query.stopSequences,
      top_k: query.topK,
      top_p: query.topP,
      max_tokens: query.maxTokens,
      temperature: query.temperature,
      tool_choice: _AnthropicRunner.mapToolChoice(query.toolsConfig),
      tools: _AnthropicRunner.mapToolConfig(query.toolsConfig),
      messages: query.messages.map(_AnthropicRunner.mapMessage)
    };
  }
  static mapToolConfig(toolConfig) {
    if (!toolConfig) return void 0;
    if (toolConfig.type === "single")
      return [_AnthropicRunner.mapTool(toolConfig.tool)];
    return toolConfig.tools.map(_AnthropicRunner.mapTool);
  }
  static mapTool(functionCall) {
    const schema = (0, import_zod_to_json_schema.zodToJsonSchema)(functionCall.schema);
    delete schema.$schema;
    return {
      name: functionCall.name,
      description: functionCall.schema?.description,
      input_schema: schema
    };
  }
  static mapToolChoice(toolConfig) {
    if (!toolConfig) return void 0;
    if (toolConfig.type === "single")
      return {
        type: "tool",
        name: toolConfig.tool.name
      };
    if (toolConfig.requireToolUse) return { type: "any" };
    return { type: "auto" };
  }
  static mapMessage(message) {
    if (message.role === "user")
      return {
        role: "user",
        content: message.content.map(_AnthropicRunner.mapMessageContent)
      };
    if (message.role === "assistant")
      return {
        role: "assistant",
        content: message.content.map(_AnthropicRunner.mapMessageContent)
      };
    never(message);
  }
  static mapMessageContent(messagePart) {
    if (messagePart.type === "text")
      return messagePart;
    if (messagePart.type === "image")
      return {
        type: "image",
        source: {
          type: messagePart.image.type,
          data: messagePart.image.data,
          media_type: messagePart.image.mimeType
        }
      };
    if (messagePart.type === "tool_call")
      return {
        type: "tool_use",
        id: messagePart.toolCallId,
        name: messagePart.toolName,
        input: messagePart.toolArgs
      };
    if (messagePart.type === "tool_response")
      return {
        type: "tool_result",
        tool_use_id: messagePart.toolCallId,
        output: messagePart.toolOutput
      };
    never(messagePart);
  }
  // #endregion
};
var runner = AnthropicRunner.new();
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  AnthropicRunner,
  runner
});
//# sourceMappingURL=anthropic.index.cjs.map