import { L as Llm, M as Merge, O as OneToN } from './llm.types-C70OLa4M.js';
import * as _synstack_resolved from '@synstack/resolved';
import { Resolvable } from '@synstack/resolved';

interface CompletionRunner<TCompletion extends Llm.Completion.Partial> {
    chatCompletion(completion: CompletionBuilder<Resolvable<TCompletion>>): Promise<{
        message: Llm.Assistant.Message;
        usage: Llm.Completion.Usage;
        stopReason: Llm.Completion.StopReason;
    }>;
    runChatCompletion<T extends TCompletion>(completion: CompletionBuilder<Resolvable<T>>): Promise<CompletionBuilder<Merge<T, Llm.Completion.Response.Part>>>;
}

declare namespace CompletionBuilder {
    type Merge<TCurrent extends Resolvable<Llm.Completion.Partial>, TNew extends Resolvable<Llm.Completion.Partial>, TCurrentValue extends Llm.Completion.Partial = Resolvable.Infer<TCurrent>, TNewValue extends Llm.Completion.Partial = Resolvable.Infer<TNew>> = TNew | TCurrent extends Promise<any> ? CompletionBuilder<Promise<{
        [K in keyof TCurrentValue | keyof TNewValue]: K extends keyof TNewValue ? TNewValue[K] : K extends keyof TCurrentValue ? TCurrentValue[K] : never;
    }>> : CompletionBuilder<{
        [K in keyof TCurrentValue | keyof TNewValue]: K extends keyof TNewValue ? TNewValue[K] : K extends keyof TCurrentValue ? TCurrentValue[K] : never;
    }>;
    type ForceValue<TValue, TForced> = Resolvable.Infer<TValue> extends undefined ? undefined : TForced;
}
declare class CompletionBuilder<T extends Resolvable<Llm.Completion.Partial>> {
    private readonly _completion;
    private constructor();
    static get new(): CompletionBuilder<{}>;
    static from<U extends Resolvable<Llm.Completion.Partial>>(completion: U): CompletionBuilder.Merge<{}, U, {}, Awaited<U>>;
    get $(): T;
    $_<R>(fn: (value: Resolvable.Infer<T>) => R): true extends Resolvable.MaybeArray.IsPromise<T> ? Resolvable.MaybeArray.IsPromise<T> & true extends Resolvable.MaybeArray.IsPromise<R> ? _synstack_resolved.Resolver<R> : _synstack_resolved.Resolver<Promise<R>> : _synstack_resolved.Resolver<R>;
    toSync(): Promise<CompletionBuilder<Resolvable.Infer<T>>>;
    /**
     * Override the values of the current builder with the provided values as object
     */
    merge<U extends Resolvable<Llm.Completion.Partial>>(completion: U): CompletionBuilder.Merge<T, U>;
    /**
     * Set the system prompt
     */
    system<U extends Resolvable<string | undefined>>(system: U): CompletionBuilder.Merge<T, {
        system: CompletionBuilder.ForceValue<U, string>;
    } | Promise<{
        system: CompletionBuilder.ForceValue<U, string>;
    }>, Awaited<T>, {
        system: CompletionBuilder.ForceValue<U, string>;
    }>;
    /**
     * Set the llm temperature
     * @argument temperature The temperature value between 0 and 1
     */
    temperature(temperature: number): CompletionBuilder.Merge<T, {
        temperature: number;
    }, Awaited<T>, {
        temperature: number;
    }>;
    /**
     * Set the maximum number of tokens to generate
     */
    maxTokens<U extends Resolvable<number>>(maxTokens: U): CompletionBuilder.Merge<T, {
        maxTokens: number;
    } | Promise<{
        maxTokens: number;
    }>, Awaited<T>, {
        maxTokens: number;
    }>;
    /**
     * Set the stop sequences.
     *
     * If the LLM generates a stop sequence, the completion will stop.
     */
    stopSequences<U extends Resolvable<OneToN<string> | undefined>>(sequences: U): CompletionBuilder.Merge<T, {
        stopSequences: Resolvable.MaybeArray.Infer<U>;
    } | Promise<{
        stopSequences: Resolvable.MaybeArray.Infer<U>;
    }>, Awaited<T>, {
        stopSequences: Resolvable.MaybeArray.Infer<U>;
    }>;
    /**
     * Set the top k tokens to consider for the completion
     */
    topK<U extends Resolvable<number | undefined>>(topK: U): CompletionBuilder.Merge<T, {
        topK: Resolvable.MaybeArray.Infer<U>;
    } | Promise<{
        topK: Resolvable.MaybeArray.Infer<U>;
    }>, Awaited<T>, {
        topK: Resolvable.MaybeArray.Infer<U>;
    }>;
    /**
     * Set the top p tokens to consider for the completion
     */
    topP<U extends Resolvable<number | undefined>>(topP: U): CompletionBuilder.Merge<T, {
        topP: Resolvable.MaybeArray.Infer<U>;
    } | Promise<{
        topP: Resolvable.MaybeArray.Infer<U>;
    }>, Awaited<T>, {
        topP: Resolvable.MaybeArray.Infer<U>;
    }>;
    /**
     * Reset the tools
     */
    clearTools(): CompletionBuilder.Merge<T, {
        toolsConfig: undefined;
    }, Awaited<T>, {
        toolsConfig: undefined;
    }>;
    /**
     * Set the tools available to the LLM
     *
     * @argument tools The tools to use
     * @argument requireToolUse Whether to require the LLM to use the tools
     */
    tools<TTools extends Resolvable<OneToN<Llm.Tool>>, TRequire extends boolean = false>(tools: TTools, requireToolUse?: TRequire): CompletionBuilder.Merge<T, {
        toolsConfig: Llm.Completion.ToolConfig.Multi<Awaited<TTools>, TRequire>;
    } | Promise<{
        toolsConfig: Llm.Completion.ToolConfig.Multi<Awaited<TTools>, TRequire>;
    }>, Awaited<T>, {
        toolsConfig: Llm.Completion.ToolConfig.Multi<Awaited<TTools>, TRequire>;
    }>;
    /**
     * Set a single tool for the LLM to use. The LLM will always respond with this tool
     */
    tool<TTool extends Resolvable<Llm.Tool>>(tool: TTool): CompletionBuilder.Merge<T, {
        toolsConfig: Llm.Completion.ToolConfig.Single<Awaited<TTool>>;
    } | Promise<{
        toolsConfig: Llm.Completion.ToolConfig.Single<Awaited<TTool>>;
    }>, Awaited<T>, {
        toolsConfig: Llm.Completion.ToolConfig.Single<Awaited<TTool>>;
    }>;
    /**
     * Set the stop reason for the completion
     */
    stopReason<U extends Llm.Completion.StopReason | undefined>(stopReason: U): CompletionBuilder.Merge<T, {
        stopReason: Resolvable.MaybeArray.Infer<U>;
    } | Promise<{
        stopReason: Resolvable.MaybeArray.Infer<U>;
    }>, Awaited<T>, {
        stopReason: Resolvable.MaybeArray.Infer<U>;
    }>;
    /**
     * Set the tokens usage
     */
    usage<U extends Resolvable<Llm.Completion.Usage | undefined>>(usage: U): CompletionBuilder.Merge<T, {
        usage: CompletionBuilder.ForceValue<U, Llm.Completion.Usage>;
    } | Promise<{
        usage: CompletionBuilder.ForceValue<U, Llm.Completion.Usage>;
    }>, Awaited<T>, {
        usage: CompletionBuilder.ForceValue<U, Llm.Completion.Usage>;
    }>;
    /**
     * Add usage to existing usage
     */
    addUsage<U extends Resolvable<Llm.Completion.Usage>>(usage: U): CompletionBuilder.Merge<T, {
        usage: {
            inputTokens: number;
            outputTokens: number;
        };
    } | Promise<{
        usage: {
            inputTokens: number;
            outputTokens: number;
        };
    }>, Awaited<T>, {
        usage: {
            inputTokens: number;
            outputTokens: number;
        };
    }>;
    /**
     * Set messages
     */
    messages(messages: Resolvable<Array<Llm.Message>> | Array<Resolvable<Llm.Message>>): CompletionBuilder.Merge<T, Promise<{
        messages: Llm.Message[];
    }>, Awaited<T>, {
        messages: Llm.Message[];
    }>;
    /**
     * Add messages
     */
    addMessages(messages: Resolvable<Array<Llm.Message>> | Array<Resolvable<Llm.Message>>): CompletionBuilder.Merge<T, {
        messages: Array<Llm.Message>;
    } | Promise<{
        messages: Array<Llm.Message>;
    }>, Awaited<T>, {
        messages: Array<Llm.Message>;
    }>;
    /**
     * Add assistant message
     */
    addAssistantMessage<U extends Resolvable<Llm.Assistant.Message>, V extends {
        messages: Array<Llm.Message>;
    }>(this: CompletionBuilder<Resolvable<V>>, message: U): CompletionBuilder.Merge<Resolvable<V>, {
        messages: Llm.Message[];
    } | Promise<{
        messages: Llm.Message[];
    }>, Awaited<V>, {
        messages: Llm.Message[];
    }>;
    /**
     * Execute completion on Llm runner
     */
    run<TRunnerCompletion extends Llm.Completion.Partial, TCompletion extends TRunnerCompletion>(this: CompletionBuilder<TCompletion>, runner: CompletionRunner<TRunnerCompletion>): Promise<CompletionBuilder<Merge<TCompletion, Llm.Completion.Response.Part>>>;
    lastMessage(): Llm.Message | Promise<Llm.Message>;
    /**
     * Get last message as text
     */
    lastMessageText(): Promise<string>;
    /**
     * Get last message tool calls
     */
    lastMessageToolCalls<C extends {
        stopReason: Llm.Completion.StopReason;
        toolsConfig: Llm.Completion.ToolConfig;
    }>(this: CompletionBuilder<C>): Llm.Message.Content.ToolCall.ResponseFromToolConfig<C["toolsConfig"]>;
}
declare const completion: CompletionBuilder<{}>;

export { CompletionBuilder as C, type CompletionRunner as a, completion as c };
